from math import *
from typing import Tuple, Dict, List
import eurocodepy as ec


def bend_ast(b: float=0.3, d: float=0.5, med: float=100.0, fck: float=20.0, fyk: float=400.0) -> Tuple[float, float, float, float]:
    """[summary]

    Args:
        b (float, optional): [description]. Defaults to 0.3.
        d (float, optional): [description]. Defaults to 0.5.
        med (float, optional): [description]. Defaults to 100.0.
        fck (float, optional): [description]. Defaults to 20.0.
        fyk (float, optional): [description]. Defaults to 400.0.

    Returns:
        Tuple[float, float, float, float]: [description]
    """
    fcd = fck/1.5
    fyd = fyk/1.15
    mmed = med
    bb = b
    dd = d
    miu = mmed/bb/d**2/fcd/1000.0
    try:
        omega = 1.0-sqrt(1-2*miu)
    except:
        omega = nan
    alpha = 1.25*omega
    ast = omega*b*d*fcd/fyd * 10000.0
    epss = (1.0-alpha)*3.5/alpha
    return ast, epss, omega, miu


def bend_ast_asc():
    return


def bend_mrd():
    return


def bend_params(conc:str='C20/25')->Tuple[float, float]:
    n = ec.ConcreteClasses[conc]['n']
    epsc2 = ec.ConcreteClasses[conc]['epsc2']
    epscu2 = ec.ConcreteClasses[conc]['epscu2']
    epsc12 = epsc2/epscu2
    chi1 = 1.0-epsc12/(n+1)
    chi2 = 1.0-((n+1)*(n+2)*0.5-epsc12**2)/((n+1)*(n+2)*chi1)
    chi = 0.5*chi1/chi2
    return chi1, chi2, chi
    


import math as m


cemprops = {
    'Type S': [3, 0.13],
    'Type N': [4, 0.12],
    'Type R': [6, 0.11],
}


def creep_coef(t=28, h0=100,rh=65, t0=10, fck=20.0, cem=0.0)->float:
    """Calculates the creep coeficient.

    Args:
        t (int, optional): _description_. Defaults to 28.
        h0 (int, optional): _description_. Defaults to 100.
        rh (int, optional): _description_. Defaults to 65.
        t0 (int, optional): _description_. Defaults to 10.
        fck (float, optional): _description_. Defaults to 20.0.
        cem (float, optional): _description_. Defaults to 0.0.

    Returns:
        float: _description_
    """
    fcm = fck+8
    alpha1 = (35/fcm)**0.7
    alpha2 = (35/fcm)**0.2
    alpha3 = min(1.0, (35/fcm)**0.5)
    tt0 = t0*((1.0+9.0/(2.0+t0**1.2))**cem)
    phi_RH = (1.0-rh/100)/(0.1*(h0**0.33333333))
    phi_RH = 1.0+phi_RH if fcm <= 35 else (1.0+phi_RH*alpha1)*alpha2
    beta_fcm = 16.8/m.sqrt(fcm)
    beta_t0 = 1.0/(0.1+tt0**0.2)
    phi_0 = beta_fcm*beta_t0*phi_RH

    try:        
        betah = min(1500*alpha3, 1.5*(1.0+m.pow(0.012*rh,18))*h0+250*alpha3)
        beta_cc = m.pow((t-t0)/(betah+t-t0), 0.3)
        phi = beta_cc*phi_0    
    except:
        beta_cc = 0.0
        phi = 0.0

    return phi


def shrink_strain(t=28, h0=100, ts=3, rh=65, fck=20.0, cem='Type N')->float:
    """_summary_

    Args:
        t (int, optional): _description_. Defaults to 28.
        h0 (int, optional): _description_. Defaults to 100.
        ts (int, optional): _description_. Defaults to 3.
        rh (int, optional): _description_. Defaults to 65.
        fck (float, optional): _description_. Defaults to 20.0.
        cem (str, optional): _description_. Defaults to 'Type N'.

    Returns:
        float: the total shrinkage strain
    """
    fcm = fck+8
    alpha1 = cemprops[cem][0]
    alpha2 = cemprops[cem][1]

    eps_ca = 25.0e-6*(fck-10)
    beta_as = 1.0-m.exp(-0.2*(t**0.5))

    beta_rh = 1.55*(1.0-(rh/100)**3)
    eps_cd = beta_rh*0.85e-6*((220+110*alpha1)*m.exp(-alpha2*fcm/10.0))
    beta_ds = (t-ts)/((t-ts)+0.4*h0**1.5)
    
    return beta_as*eps_ca + beta_ds*eps_cd


import json
import os
import pandas as pd
import importlib.resources as pkg_resources

database = {}
db = json.loads(open(os.path.join(os.path.dirname(__file__),'eurocodes.json'),'r').read())["Eurocodes"]
db["SteelProfiles"]["Euro"] = json.loads(open(os.path.join(os.path.dirname(__file__),'euro.prof.json'),'r').read())


# Turns a dictionary into a class
class Dict2Class(object):

    def __init__(self, my_dict):
        for key in my_dict:
            setattr(self, key, my_dict[key])

dbase = Dict2Class(db)

def _get_database() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    f = pkg_resources.open_text(__package__, 'eurocodes.json')
    #f = open(filename, "r")
    database = json.loads(f.read())
    return database


def _get_database2() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    f = open(os.path.join(os.path.dirname(__file__),'eurocodes.json'),'r')
    #f = open(filename, "r")
    database = json.loads(f.read())
    return database


def get_eurocodes() -> dict:
    """
    """
    return _get_database2()["Eurocodes"]


def get_materials() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    database = _get_database2()
    return database["Eurocodes"]["Materials"]


def get_timber() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    database = _get_database2()
    return database["Eurocodes"]["Materials"]["Timber"]


def get_concrete() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    database = _get_database2()
    return database["Eurocodes"]["Materials"]["Concrete"]


def get_prestress() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    database = _get_database2()
    return database["Eurocodes"]["Materials"]["Prestress"]


def get_reinforcement() -> dict:
    """[summary]
    Returns:
        dict: [description]
    """
    global database
    database = _get_database2()
    return database["Eurocodes"]["Materials"]["Reinforcement"]

ConcreteClasses = db["Materials"]["Concrete"]["Classes"]
PrestressClasses = db["Materials"]["Prestress"]["Classes"]
ReinforcementClasses = db["Materials"]["Reinforcement"]["Classes"]
ReinforcementBars = db["Materials"]["Reinforcement"]["Rebars"]
SteelProofiles = db["SteelProfiles"]["Euro"]


from math import log

c_season = 1.0
c_dir = 1.0
k_1 = 1.0 # coeficiente de turbulência
rho = 1.25 # kg/m3
z0 = {"0": 0.003, "I": 0.01, "II": 0.05, "III": 0.3, "IV": 1}
zmin = {"0": 1, "I": 1, "II": 2, "III": 5, "IV": 10}


def v_b(vb_0: float) -> float:
    """Calculates the basic wind velocity 

    Args:
        vb_0 (float): fundamental value of the basic wind velocity

    Returns:
        float: basic wind velocity 
    """
    return c_season * c_dir * vb_0


def c_r(z: float, zone: str) -> float:
    """ Calculate the roughness factor

    Args:
        z (float): vertical distance
        zone (str): the terrain category

    Returns:
        float: the roughness factor
    """
    k_r = 0.19*((z0[zone]/z0["II"])**0.07)
    zeff = z if z >= zmin[zone] else zmin[zone]
    return k_r * log(zeff/z0[zone])


def c_0(z: float) -> float:
    """Calculates the orography factor, taken as 1,0
    Args:
        z (float): vertical distance

    Returns:
        float: orography factor
    """
    return 1.0


def v_m(z: float, vb: float, zone: str) -> float:
    """ Calculates the mean wind velocity, vm(z), at a height z above the terrain.
    Depends on the terrain roughness and orography and on the basic wind velocity.

    Args:
        z (float): vertical distance
        vb (float): basic wind velocity
        zone (str): the terrain category

    Returns:
        float: mean wind velocity, vm(z) 
    """
    return c_r(z, zone) * c_0(z) * vb


def I_v(z: float, zone: str) -> float:
    """Calculates the turbulence intensity, Iv(z), at height z.
    It is defined as the standard deviation of the turbulence divided by the mean wind velocity.

    Args:
        z (float): vertical distance
        zone (str): the terrain category

    Returns:
        float: turbulence intensity
    """
    zeff = z if z >= zmin[zone] else zmin[zone]
    Iv = k_1 / c_0(z) / log(zeff/z0[zone])
    return Iv


def q_p(z: float, vb0: float, zone: str) -> float:
    """Calcculates the peak velocity pressure, qp(z), at height z, 
    which includes mean and short-term velocity fluctuations.

    Args:
        z (float): vertical distance
        vb0 (float): fundamental value of the basic wind velocity
        zone (str): the terrain category

    Returns:
        float: peak velocity pressure
    """
    zone = str.upper(zone)
    v = v_m(z, v_b(vb0), zone)
    #v = c_r(z, zone) * c_0(z) * vb0
    qp = 0.5 * (1.0 + 7*I_v(z, zone)) * v**2 * rho
    return qp


from . import fire
from . import sls
from . import uls

from .sls.longterm import cemprops
from .sls.longterm import creep_coef
from .sls.longterm import shrink_strain
# from .uls.shear import *
# from .uls.bend_simple import *

from .uls.shear import shear_asws
from .uls.shear import shear_vrd
from .uls.shear import shear_vrdc
from .uls.bend_simple import bend_params
# from .ec2_crack import *
# from .ec2_deformation import *


from .db import get_timber
from .db import get_concrete
from .db import get_prestress
from .db import get_reinforcement
from .db import get_materials
from .db import get_eurocodes
from .db import PrestressClasses
from .db import ReinforcementBars
from .db import ReinforcementClasses
from .db import ConcreteClasses

from .db import db
from .db import dbase as db2

from . import ec2
from . import ec5
from . import utils
